@@ -1,7 +1,7 @@
 /**************************************************************************
  *   move.c  --  This file is part of GNU nano.                           *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2019 Free Software Foundation, Inc.    *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
  *   Copyright (C) 2014-2018 Benno Schulenberg                            *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
@@ -19,7 +19,7 @@
  *                                                                        *
  **************************************************************************/
 
-#include "proto.h"
+#include "prototypes.h"
 
 #include <string.h>
 
@@ -37,7 +37,7 @@ void to_first_line(void)
 void to_last_line(void)
 {
 	openfile->current = openfile->filebot;
-	openfile->current_x = strlen(openfile->filebot->data);
+	openfile->current_x = (inhelp) ? 0 : strlen(openfile->filebot->data);
 	openfile->placewewant = xplustabs();
 
 	/* Set the last line of the screen as the target for the cursor. */
@@ -109,12 +109,13 @@ void set_proper_index_and_pww(size_t *leftedge, size_t target, bool forward)
 	openfile->placewewant = *leftedge + target;
 }
 
-/* Move up nearly one screenful. */
+/* Move up almost one screenful. */
 void do_page_up(void)
 {
 	int mustmove = (editwinrows < 3) ? 1 : editwinrows - 2;
 	size_t leftedge, target_column;
 
+#ifndef NANO_TINY
 	/* If we're not in smooth scrolling mode, put the cursor at the
 	 * beginning of the top line of the edit window, as Pico does. */
 	if (ISSET(JUMPY_SCROLLING)) {
@@ -123,6 +124,7 @@ void do_page_up(void)
 		openfile->current_y = 0;
 		target_column = 0;
 	} else
+#endif
 		get_edge_and_target(&leftedge, &target_column);
 
 	/* Move up the required number of lines or chunks.  If we can't, we're
@@ -139,12 +141,13 @@ void do_page_up(void)
 	refresh_needed = TRUE;
 }
 
-/* Move down nearly one screenful. */
+/* Move down almost one screenful. */
 void do_page_down(void)
 {
 	int mustmove = (editwinrows < 3) ? 1 : editwinrows - 2;
 	size_t leftedge, target_column;
 
+#ifndef NANO_TINY
 	/* If we're not in smooth scrolling mode, put the cursor at the
 	 * beginning of the top line of the edit window, as Pico does. */
 	if (ISSET(JUMPY_SCROLLING)) {
@@ -153,6 +156,7 @@ void do_page_down(void)
 		openfile->current_y = 0;
 		target_column = 0;
 	} else
+#endif
 		get_edge_and_target(&leftedge, &target_column);
 
 	/* Move down the required number of lines or chunks.  If we can't, we're
@@ -192,7 +196,7 @@ void do_para_end(linestruct **line)
 }
 
 /* Move up to first start of a paragraph before the current line. */
-void do_para_begin_void(void)
+void to_para_begin(void)
 {
 	linestruct *was_current = openfile->current;
 
@@ -203,7 +207,7 @@ void do_para_begin_void(void)
 }
 
 /* Move down to just after the first found end of a paragraph. */
-void do_para_end_void(void)
+void to_para_end(void)
 {
 	linestruct *was_current = openfile->current;
 
@@ -222,7 +226,7 @@ void do_para_end_void(void)
 #endif /* ENABLE_JUSTIFY */
 
 /* Move to the preceding block of text. */
-void do_prev_block(void)
+void to_prev_block(void)
 {
 	linestruct *was_current = openfile->current;
 	bool is_text = FALSE, seen_text = FALSE;
@@ -234,8 +238,8 @@ void do_prev_block(void)
 		seen_text = seen_text || is_text;
 	}
 
-	/* Step forward one line again if this one is blank. */
-	if (openfile->current->next != NULL &&
+	/* Step forward one line again if we passed text but this line is blank. */
+	if (seen_text && openfile->current->next != NULL &&
 				white_string(openfile->current->data))
 		openfile->current = openfile->current->next;
 
@@ -244,7 +248,7 @@ void do_prev_block(void)
 }
 
 /* Move to the next block of text. */
-void do_next_block(void)
+void to_next_block(void)
 {
 	linestruct *was_current = openfile->current;
 	bool is_white = white_string(openfile->current->data);
@@ -278,15 +282,19 @@ void do_prev_word(bool allow_punct)
 		}
 
 		/* Step back one character. */
-		openfile->current_x = move_mbleft(openfile->current->data,
+		openfile->current_x = step_left(openfile->current->data,
 												openfile->current_x);
 
-		if (is_word_mbchar(openfile->current->data + openfile->current_x,
+		if (is_word_char(openfile->current->data + openfile->current_x,
 								allow_punct)) {
 			seen_a_word = TRUE;
 			/* If at the head of a line now, this surely is a word start. */
 			if (openfile->current_x == 0)
 				break;
+#ifdef ENABLE_UTF8
+		} else if (is_zerowidth(openfile->current->data + openfile->current_x)) {
+			; /* skip */
+#endif
 		} else if (seen_a_word) {
 			/* This is space now: we've overshot the start of the word. */
 			step_forward = TRUE;
@@ -296,7 +304,7 @@ void do_prev_word(bool allow_punct)
 
 	if (step_forward)
 		/* Move one character forward again to sit on the start of the word. */
-		openfile->current_x = move_mbright(openfile->current->data,
+		openfile->current_x = step_right(openfile->current->data,
 												openfile->current_x);
 }
 
@@ -305,7 +313,7 @@ void do_prev_word(bool allow_punct)
  * part of a word.  Return TRUE if we started on a word, and FALSE otherwise. */
 bool do_next_word(bool after_ends, bool allow_punct)
 {
-	bool started_on_word = is_word_mbchar(openfile->current->data +
+	bool started_on_word = is_word_char(openfile->current->data +
 								openfile->current_x, allow_punct);
 	bool seen_space = !started_on_word;
 #ifndef NANO_TINY
@@ -324,7 +332,7 @@ bool do_next_word(bool after_ends, bool allow_punct)
 			seen_space = TRUE;
 		} else {
 			/* Step forward one character. */
-			openfile->current_x = move_mbright(openfile->current->data,
+			openfile->current_x = step_right(openfile->current->data,
 												openfile->current_x);
 		}
 
@@ -332,17 +340,26 @@ bool do_next_word(bool after_ends, bool allow_punct)
 		if (after_ends) {
 			/* If this is a word character, continue; else it's a separator,
 			 * and if we've already seen a word, then it's a word end. */
-			if (is_word_mbchar(openfile->current->data + openfile->current_x,
+			if (is_word_char(openfile->current->data + openfile->current_x,
 								allow_punct))
 				seen_word = TRUE;
+#ifdef ENABLE_UTF8
+			else if (is_zerowidth(openfile->current->data + openfile->current_x))
+				; /* skip */
+#endif
 			else if (seen_word)
 				break;
 		} else
 #endif
 		{
+#ifdef ENABLE_UTF8
+			if (is_zerowidth(openfile->current->data + openfile->current_x))
+				; /* skip */
+			else
+#endif
 			/* If this is not a word character, then it's a separator; else
 			 * if we've already seen a separator, then it's a word start. */
-			if (!is_word_mbchar(openfile->current->data + openfile->current_x,
+			if (!is_word_char(openfile->current->data + openfile->current_x,
 								allow_punct))
 				seen_space = TRUE;
 			else if (seen_space)
@@ -355,7 +372,7 @@ bool do_next_word(bool after_ends, bool allow_punct)
 
 /* Move to the previous word in the file, treating punctuation as part of a
  * word if the WORD_BOUNDS flag is set, and update the screen afterwards. */
-void do_prev_word_void(void)
+void to_prev_word(void)
 {
 	linestruct *was_current = openfile->current;
 
@@ -367,7 +384,7 @@ void do_prev_word_void(void)
 /* Move to the next word in the file.  If the AFTER_ENDS flag is set, stop
  * at word ends instead of beginnings.  If the WORD_BOUNDS flag is set, treat
  * punctuation as part of a word.  Update the screen afterwards. */
-void do_next_word_void(void)
+void to_next_word(void)
 {
 	linestruct *was_current = openfile->current;
 
@@ -437,7 +454,7 @@ void do_home(void)
 }
 
 /* Move to the end of the current line (or softwrapped chunk).
- * When softwrapping and alredy at the end of a chunk, go to the
+ * When softwrapping and already at the end of a chunk, go to the
  * end of the full line. */
 void do_end(void)
 {
@@ -534,7 +551,7 @@ void do_down(void)
 }
 
 #if !defined(NANO_TINY) || defined(ENABLE_HELP)
-/* Scroll up one line or chunk without scrolling the cursor. */
+/* Scroll up one line or chunk without moving the cursor textwise. */
 void do_scroll_up(void)
 {
 	/* When the top of the file is onscreen, we can't scroll. */
@@ -548,31 +565,45 @@ void do_scroll_up(void)
 		edit_scroll(BACKWARD);
 }
 
-/* Scroll down one line or chunk without scrolling the cursor. */
+/* Scroll down one line or chunk without moving the cursor textwise. */
 void do_scroll_down(void)
 {
 	if (openfile->current_y == 0)
 		do_down();
 
-	if (openfile->edittop->next != NULL
+	if (editwinrows > 1 && (openfile->edittop->next != NULL
 #ifndef NANO_TINY
-				|| chunk_for(openfile->firstcolumn, openfile->edittop) <
-					number_of_chunks_in(openfile->edittop)
+				|| (ISSET(SOFTWRAP) && (extra_chunks_in(openfile->edittop) >
+					chunk_for(openfile->firstcolumn, openfile->edittop)))
 #endif
-										)
+										))
 		edit_scroll(FORWARD);
 }
-#endif
+
+/* Scroll the line with the cursor to the center of the screen. */
+void do_center(void)
+{
+	adjust_viewport(CENTERING);
+	draw_all_subwindows();
+	full_refresh();
+}
+#endif /* !NANO_TINY || ENABLE_HELP */
 
 /* Move left one character. */
 void do_left(void)
 {
 	linestruct *was_current = openfile->current;
 
-	if (openfile->current_x > 0)
-		openfile->current_x = move_mbleft(openfile->current->data,
+	if (openfile->current_x > 0) {
+		openfile->current_x = step_left(openfile->current->data,
 												openfile->current_x);
-	else if (openfile->current != openfile->filetop) {
+#ifdef ENABLE_UTF8
+		while (openfile->current_x > 0 &&
+					is_zerowidth(openfile->current->data + openfile->current_x))
+			openfile->current_x = step_left(openfile->current->data,
+												openfile->current_x);
+#endif
+	} else if (openfile->current != openfile->filetop) {
 		openfile->current = openfile->current->prev;
 		openfile->current_x = strlen(openfile->current->data);
 	}
@@ -585,10 +616,16 @@ void do_right(void)
 {
 	linestruct *was_current = openfile->current;
 
-	if (openfile->current->data[openfile->current_x] != '\0')
-		openfile->current_x = move_mbright(openfile->current->data,
+	if (openfile->current->data[openfile->current_x] != '\0') {
+		openfile->current_x = step_right(openfile->current->data,
 												openfile->current_x);
-	else if (openfile->current != openfile->filebot) {
+#ifdef ENABLE_UTF8
+		while (openfile->current->data[openfile->current_x] != '\0' &&
+					is_zerowidth(openfile->current->data + openfile->current_x))
+			openfile->current_x = step_right(openfile->current->data,
+												openfile->current_x);
+#endif
+	} else if (openfile->current != openfile->filebot) {
 		openfile->current = openfile->current->next;
 		openfile->current_x = 0;
 	}
