@@ -1,8 +1,8 @@
 /**************************************************************************
  *   prompt.c  --  This file is part of GNU nano.                         *
  *                                                                        *
- *   Copyright (C) 1999-2011, 2013-2020 Free Software Foundation, Inc.    *
- *   Copyright (C) 2016, 2018 Benno Schulenberg                           *
+ *   Copyright (C) 1999-2011, 2013-2021 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2016, 2018, 2020 Benno Schulenberg                     *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -19,7 +19,7 @@
  *                                                                        *
  **************************************************************************/
 
-#include "proto.h"
+#include "prototypes.h"
 
 #include <string.h>
 
@@ -57,9 +57,18 @@ void do_statusbar_next_word(void)
 			 * and if we've already seen a word, then it's a word end. */
 			if (is_word_char(answer + typing_x, FALSE))
 				seen_word = TRUE;
+#ifdef ENABLE_UTF8
+			else if (is_zerowidth(answer + typing_x))
+				; /* skip */
+#endif
 			else if (seen_word)
 				break;
 		} else {
+#ifdef ENABLE_UTF8
+			if (is_zerowidth(answer + typing_x))
+				; /* skip */
+			else
+#endif
 			/* If this is not a word character, then it's a separator; else
 			 * if we've already seen a separator, then it's a word start. */
 			if (!is_word_char(answer + typing_x, FALSE))
@@ -81,6 +90,10 @@ void do_statusbar_prev_word(void)
 
 		if (is_word_char(answer + typing_x, FALSE))
 			seen_a_word = TRUE;
+#ifdef ENABLE_UTF8
+		else if (is_zerowidth(answer + typing_x))
+			; /* skip */
+#endif
 		else if (seen_a_word) {
 			/* This is space now: we've overshot the start of the word. */
 			step_forward = TRUE;
@@ -97,15 +110,25 @@ void do_statusbar_prev_word(void)
 /* Move left one character in the answer. */
 void do_statusbar_left(void)
 {
-	if (typing_x > 0)
+	if (typing_x > 0) {
 		typing_x = step_left(answer, typing_x);
+#ifdef ENABLE_UTF8
+		while (typing_x > 0 && is_zerowidth(answer + typing_x))
+			typing_x = step_left(answer, typing_x);
+#endif
+	}
 }
 
 /* Move right one character in the answer. */
 void do_statusbar_right(void)
 {
-	if (answer[typing_x] != '\0')
+	if (answer[typing_x] != '\0') {
 		typing_x = step_right(answer, typing_x);
+#ifdef ENABLE_UTF8
+		while (answer[typing_x] != '\0' && is_zerowidth(answer + typing_x))
+			typing_x = step_right(answer, typing_x);
+#endif
+	}
 }
 
 /* Delete one character in the answer. */
@@ -116,6 +139,10 @@ void do_statusbar_delete(void)
 
 		memmove(answer + typing_x, answer + typing_x + charlen,
 						strlen(answer) - typing_x - charlen + 1);
+#ifdef ENABLE_UTF8
+		if (is_zerowidth(answer + typing_x))
+			do_statusbar_delete();
+#endif
 	}
 }
 
@@ -123,8 +150,10 @@ void do_statusbar_delete(void)
 void do_statusbar_backspace(void)
 {
 	if (typing_x > 0) {
+		size_t was_x = typing_x;
+
 		typing_x = step_left(answer, typing_x);
-		do_statusbar_delete();
+		memmove(answer + typing_x, answer + was_x, strlen(answer) - was_x + 1);
 	}
 }
 
@@ -141,7 +170,7 @@ void do_statusbar_cut_text(void)
 void paste_into_answer(void)
 {
 	size_t pastelen = strlen(cutbuffer->data);
-	char *fusion = charalloc(strlen(answer) + pastelen + 1);
+	char *fusion = nmalloc(strlen(answer) + pastelen + 1);
 
 	/* Concatenate: the current answer before the cursor, the first line
 	 * of the cutbuffer, plus the rest of the current answer. */
@@ -184,7 +213,7 @@ void inject_into_answer(char *burst, size_t count)
 		if (burst[index] == '\0')
 			burst[index] = '\n';
 
-	answer = charealloc(answer, strlen(answer) + count + 1);
+	answer = nrealloc(answer, strlen(answer) + count + 1);
 	memmove(answer + typing_x + count, answer + typing_x,
 								strlen(answer) - typing_x + 1);
 	strncpy(answer + typing_x, burst , count);
@@ -195,12 +224,15 @@ void inject_into_answer(char *burst, size_t count)
 /* Get a verbatim keystroke and insert it into the answer. */
 void do_statusbar_verbatim_input(void)
 {
+	size_t count = 1;
 	char *bytes;
-	size_t count;
 
 	bytes = get_verbatim_kbinput(bottomwin, &count);
 
-	inject_into_answer(bytes, count);
+	if (0 < count && count < 999)
+		inject_into_answer(bytes, count);
+	else if (count == 0)
+		beep();
 
 	free(bytes);
 }
@@ -250,7 +282,7 @@ int do_statusbar_input(bool *finished)
 			beep();
 		else if (!ISSET(RESTRICTED) || currmenu != MWRITEFILE ||
 						openfile->filename[0] == '\0') {
-			puddle = charealloc(puddle, depth + 2);
+			puddle = nrealloc(puddle, depth + 2);
 			puddle[depth++] = (char)input;
 		}
 	}
@@ -348,18 +380,19 @@ void put_cursor_at_end_of_answer(void)
 	typing_x = HIGHEST_POSITIVE;
 }
 
-/* Redraw the promptbar and place the cursor at the right spot. */
+/* Redraw the prompt bar and place the cursor at the right spot. */
 void draw_the_promptbar(void)
 {
 	size_t base = breadth(prompt) + 2;
-	size_t the_page, end_page, column;
+	size_t column = base + wideness(answer, typing_x);
+	size_t the_page, end_page;
 	char *expanded;
 
-	the_page = get_statusbar_page_start(base, base + wideness(answer, typing_x));
+	the_page = get_statusbar_page_start(base, column);
 	end_page = get_statusbar_page_start(base, base + breadth(answer) - 1);
 
-	/* Color the promptbar over its full width. */
-	wattron(bottomwin, interface_color_pair[TITLE_BAR]);
+	/* Color the prompt bar over its full width. */
+	wattron(bottomwin, interface_color_pair[PROMPT_BAR]);
 	mvwprintw(bottomwin, 0, 0, "%*s", COLS, " ");
 
 	mvwaddstr(bottomwin, 0, 0, prompt);
@@ -370,18 +403,19 @@ void draw_the_promptbar(void)
 	waddstr(bottomwin, expanded);
 	free(expanded);
 
-	if (base + breadth(answer) != COLS && the_page < end_page)
+	if (the_page < end_page && base + breadth(answer) - the_page > COLS)
 		mvwaddch(bottomwin, 0, COLS - 1, '>');
 
-	wattroff(bottomwin, interface_color_pair[TITLE_BAR]);
+	wattroff(bottomwin, interface_color_pair[PROMPT_BAR]);
 
-	/* Work around a cursor-misplacement bug in VTEs. */
-	wmove(bottomwin, 0, 0);
-	wrefresh(bottomwin);
+	/* Work around a cursor-misplacement bug -- https://sv.gnu.org/bugs/?59808. */
+	if (ISSET(NO_HELP)) {
+		wmove(bottomwin, 0, 0);
+		wrefresh(bottomwin);
+	}
 
 	/* Place the cursor at the right spot. */
-	column = base + wideness(answer, typing_x);
-	wmove(bottomwin, 0, column - get_statusbar_page_start(base, column));
+	wmove(bottomwin, 0, column - the_page);
 	wnoutrefresh(bottomwin);
 }
 
@@ -394,7 +428,7 @@ void add_or_remove_pipe_symbol_from_answer(void)
 		if (typing_x > 0)
 			typing_x--;
 	} else {
-		answer = charealloc(answer, strlen(answer) + 2);
+		answer = nrealloc(answer, strlen(answer) + 2);
 		memmove(answer + 1, answer, strlen(answer) + 1);
 		answer[0] = '|';
 		typing_x++;
@@ -403,28 +437,22 @@ void add_or_remove_pipe_symbol_from_answer(void)
 #endif
 
 /* Get a string of input at the status-bar prompt. */
-functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
-		bool allow_files, bool *listed, linestruct **history_list,
-		void (*refresh_func)(void))
+functionptrtype acquire_an_answer(int *actual, bool *listed,
+					linestruct **history_list, void (*refresh_func)(void))
 {
 	int kbinput = ERR;
 	bool finished;
 	functionptrtype func;
-#ifdef ENABLE_TABCOMP
-	bool tabbed = FALSE;
-		/* Whether we've pressed Tab. */
-#endif
 #ifdef ENABLE_HISTORIES
 	char *history = NULL;
 		/* The current history string. */
 	char *magichistory = NULL;
 		/* The (partial) answer that was typed at the prompt, if any. */
 #ifdef ENABLE_TABCOMP
-	int last_kbinput = ERR;
-		/* The key we pressed before the current key. */
-	size_t complete_len = 0;
-		/* The length of the original string that we're trying to
-		 * tab complete, if any. */
+	bool previous_was_tab = FALSE;
+		/* Whether the previous keystroke was an attempt at tab completion. */
+	size_t fragment_length = 0;
+		/* The length of the fragment that the user tries to tab complete. */
 #endif
 #endif /* ENABLE_HISTORIES */
 
@@ -454,25 +482,22 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
 			break;
 
 #ifdef ENABLE_TABCOMP
-		if (func != do_tab)
-			tabbed = FALSE;
-
 		if (func == do_tab) {
 #ifdef ENABLE_HISTORIES
 			if (history_list != NULL) {
-				if (last_kbinput != the_code_for(do_tab, '\t'))
-					complete_len = strlen(answer);
+				if (!previous_was_tab)
+					fragment_length = strlen(answer);
 
-				if (complete_len > 0) {
+				if (fragment_length > 0) {
 					answer = get_history_completion(history_list,
-										answer, complete_len);
+													answer, fragment_length);
 					typing_x = strlen(answer);
 				}
 			} else
 #endif
-			if (allow_tabs)
-				answer = input_tab(answer, allow_files, &typing_x,
-										&tabbed, refresh_func, listed);
+			/* Allow tab completion of filenames, but not in restricted mode. */
+			if ((currmenu & (MINSERTFILE|MWRITEFILE|MGOTODIR)) && !ISSET(RESTRICTED))
+				answer = input_tab(answer, &typing_x, refresh_func, listed);
 		} else
 #endif /* ENABLE_TABCOMP */
 #ifdef ENABLE_HISTORIES
@@ -509,17 +534,20 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
 			}
 		} else
 #endif /* ENABLE_HISTORIES */
-		if (func == do_help) {
-			/* This key has a shortcut-list entry when it's used to go to
-			 * the help viewer or display a message indicating that help
-			 * is disabled, which means that finished has been set to TRUE.
-			 * Set it back to FALSE here, so that we aren't kicked out of
-			 * the status-bar prompt. */
+		/* If we ran a function that should not exit from the prompt... */
+		if (func == do_help || func == full_refresh)
 			finished = FALSE;
-		}
 #ifndef NANO_TINY
 		else if (func == do_nothing)
 			finished = FALSE;
+		else if (func == do_toggle_void) {
+			TOGGLE(NO_HELP);
+			window_init();
+			focusing = FALSE;
+			refresh_func();
+			bottombars(currmenu);
+			finished = FALSE;
+		}
 #endif
 
 		/* If we have a shortcut with an associated function, break out if
@@ -528,7 +556,7 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
 			break;
 
 #if defined(ENABLE_HISTORIES) && defined(ENABLE_TABCOMP)
-		last_kbinput = kbinput;
+		previous_was_tab = (func == do_tab);
 #endif
 	}
 
@@ -547,15 +575,10 @@ functionptrtype acquire_an_answer(int *actual, bool allow_tabs,
 
 /* Ask a question on the status bar.  Return 0 when text was entered,
  * -1 for a cancelled entry, -2 for a blank string, and the relevant
- * keycode when a valid shortcut key was pressed.
- *
- * The allow_tabs parameter indicates whether tab completion is allowed,
- * and allow_files indicates whether all files (and not just directories)
- * can be tab completed.  The 'provided' parameter is the default answer
- * for when simply Enter is typed. */
-int do_prompt(bool allow_tabs, bool allow_files,
-		int menu, const char *provided, linestruct **history_list,
-		void (*refresh_func)(void), const char *msg, ...)
+ * keycode when a valid shortcut key was pressed.  The 'provided'
+ * parameter is the default answer for when simply Enter is typed. */
+int do_prompt(int menu, const char *provided, linestruct **history_list,
+				void (*refresh_func)(void), const char *msg, ...)
 {
 	va_list ap;
 	int retval;
@@ -573,15 +596,16 @@ int do_prompt(bool allow_tabs, bool allow_files,
 #ifndef NANO_TINY
   redo_theprompt:
 #endif
-	prompt = charalloc((COLS * MAXCHARLEN) + 1);
+	prompt = nmalloc((COLS * MAXCHARLEN) + 1);
 	va_start(ap, msg);
 	vsnprintf(prompt, COLS * MAXCHARLEN, msg, ap);
 	va_end(ap);
 	/* Reserve five columns for colon plus angles plus answer, ":<aa>". */
 	prompt[actual_x(prompt, (COLS < 5) ? 0 : COLS - 5)] = '\0';
 
-	func = acquire_an_answer(&retval, allow_tabs, allow_files, &listed,
-								history_list, refresh_func);
+	lastmessage = VACUUM;
+
+	func = acquire_an_answer(&retval, &listed, history_list, refresh_func);
 	free(prompt);
 	prompt = saved_prompt;
 
@@ -602,11 +626,11 @@ int do_prompt(bool allow_tabs, bool allow_files,
 	else if (func == do_enter)
 		retval = (*answer == '\0') ? -2 : 0;
 
-	wipe_statusbar();
+	if (lastmessage == VACUUM)
+		wipe_statusbar();
 
 #ifdef ENABLE_TABCOMP
-	/* If we've done tab completion, there might still be a list of
-	 * filename matches on the edit window.  Clear them off. */
+	/* If possible filename completions are still listed, clear them off. */
 	if (listed)
 		refresh_func();
 #endif
@@ -665,11 +689,11 @@ int do_yesno_prompt(bool all, const char *msg)
 			post_one_key(cancelshortcut->keystr, _("Cancel"), width);
 		}
 
-		/* Color the promptbar over its full width and display the question. */
-		wattron(bottomwin, interface_color_pair[TITLE_BAR]);
+		/* Color the prompt bar over its full width and display the question. */
+		wattron(bottomwin, interface_color_pair[PROMPT_BAR]);
 		mvwprintw(bottomwin, 0, 0, "%*s", COLS, " ");
 		mvwaddnstr(bottomwin, 0, 0, msg, actual_x(msg, COLS - 1));
-		wattroff(bottomwin, interface_color_pair[TITLE_BAR]);
+		wattroff(bottomwin, interface_color_pair[PROMPT_BAR]);
 		wnoutrefresh(bottomwin);
 
 		currmenu = MYESNO;
@@ -678,6 +702,9 @@ int do_yesno_prompt(bool all, const char *msg)
 		kbinput = get_kbinput(bottomwin, !all);
 
 #ifndef NANO_TINY
+		if (kbinput == KEY_WINCH)
+			continue;
+
 		/* Accept the first character of an external paste. */
 		if (bracketed_paste && kbinput == BRACKETED_PASTE_MARKER)
 			kbinput = get_kbinput(bottomwin, BLIND);
@@ -738,6 +765,17 @@ int do_yesno_prompt(bool all, const char *msg)
 			}
 		}
 #endif /* ENABLE_MOUSE */
+		else if (func_from_key(&kbinput) == full_refresh)
+			full_refresh();
+#ifndef NANO_TINY
+		else if (func_from_key(&kbinput) == do_toggle_void) {
+			TOGGLE(NO_HELP);
+			window_init();
+			titlebar(NULL);
+			focusing = FALSE;
+			edit_refresh();
+		}
+#endif
 		else
 			beep();
 
